<?php

/**
 * Class Licentia_Fidelitas_Model_Subscribers
 */
class Licentia_Fidelitas_Model_Subscribers extends Mage_Core_Model_Abstract
{
    /**
     *
     */
    protected function _construct()
    {
        $this->_init('fidelitas/subscribers');
    }

    /**
     * @return bool
     * @throws Exception
     */
    function cron()
    {

        ini_set('max_execution_time', 18000);

        $day = '';
        $start = 0;
        $limit = 1000;
        $hour = 0;
        $file = Mage::getBaseDir('log') . '/egoi.cron.log';

        if (is_file($file)) {
            $data = explode('-', file_get_contents($file));
            $day = (int)@$data[0];
            $hour = (int)@$data[1];
            $start = (int)@$data[2];
        }

        if ($day == date('d') && date('H') == $hour) {
            return true;
        }

        Mage::log('started', null, 'egoi-sync-subs.log', true);

        if (Mage::getStoreConfigFlag('fidelitas/config/previous')) {
            $this->importCoreNewsletterSubscribers();
        }

        $table = $this->getResource()->getMainTable();

        $tmpClean = 1;

        try {

            Mage::getSingleton('core/resource')
                ->getConnection('core_write')
                ->query("DELETE FROM $table WHERE LENGTH(list)=0");

            while (true) {
                if ($tmpClean > 100) {
                    break;
                }

                $clean = Mage::getSingleton('core/resource')->getConnection('core_write')
                    ->fetchAll("SELECT n1.* FROM $table n1, $table n2 WHERE n1.subscriber_id > n2.subscriber_id AND n1.email = n2.email LIMIT 100");

                foreach ($clean as $item) {
                    Mage::getModel('fidelitas/subscribers')->load($item['subscriber_id'])->delete();
                }

                if (count($clean) == 0) {
                    break;
                }

                $tmpClean++;
            }

        } catch (Exception $e) {
            echo $e->getMessage();
            die();
        }

        $subscribers = $this->getCollection()
            ->addFieldToFilter('cellphone', '')
            ->addFieldToFilter('email', '');

        foreach ($subscribers as $subscriber) {
            $subscriber->delete();
        }

        $egoi = Mage::getModel('fidelitas/egoi');
        $list = Mage::getModel('fidelitas/lists')->getList();

        $end = false;
        do {
            $egoi->addData(array('listID' => $list->getListnum(), 'subscriber' => 'all_subscribers', 'limit' => $limit))
                ->setData('start', $start);

            file_put_contents($file, date('d') . '-' . date('H') . '-' . $start);

            try {
                $subscribers = $egoi->getSubscriberData()->getData();
            } catch (Exception $e) {
                Mage::logException($e);
            }


            if (isset($subscribers[0]['ERROR'])) {
                break;
            }

            if (count($subscribers[0]['subscriber']) < $limit) {
                $end = true;
            }

            foreach ($subscribers[0]['subscriber'] as $subscriber) {

                $subscriberData = array_change_key_case($subscriber, CASE_LOWER);

                if (!filter_var($subscriberData['email'], FILTER_VALIDATE_EMAIL)) {

                    $deleteInvalid = Mage::getModel('fidelitas/subscribers')->load($subscriberData['uid'], 'uid');
                    if ($deleteInvalid->getId()) {
                        $deleteInvalid->delete();
                    }

                    continue;
                }

                $jaExiste = Mage::getModel('fidelitas/subscribers')->load($subscriberData['email'], 'email');
                if ($jaExiste->getId()) {
                    $subscriberData['subscriber_id'] = $jaExiste->getId();
                }

                if (isset($subscriberData['store_id']) && !Mage::getStoreConfigFlag('fidelitas/config/enabled')) {
                    continue;
                }

                try {

                    Mage::getModel('fidelitas/subscribers')
                        ->setData($subscriberData)
                        ->setData('inCron', true)
                        ->save();

                } catch (Exception $e) {
                    Mage::logException($e);
                }
            }

            $start += $limit;

        } while ($end === false);

        Mage::log('end', null, 'egoi-sync-subs.log', true);


        unlink($file);


    }

    /**
     * @param        $value
     * @param string $attribute
     * @param null   $billing
     *
     * @return bool
     */
    public function findCustomer($value, $attribute = 'entity_id', $billing = null)
    {

        $cellphoneField = Mage::getStoreConfig('fidelitas/config/cellphone');
        $customers = Mage::getModel('customer/customer')
            ->getCollection()
            ->addAttributeToSelect('firstname')
            ->addAttributeToSelect('lastname')
            ->addAttributeToSelect('store_id')
            ->joinAttribute('dob', 'customer/dob', 'entity_id', null, 'left')
            ->joinAttribute('gender', 'customer/gender', 'entity_id', null, 'left')
            ->addAttributeToFilter($attribute, $value)
            ->joinAttribute('country_id', 'customer_address/country_id', 'default_billing', null, 'left')
            ->joinAttribute($cellphoneField, 'customer_address/' . $cellphoneField, 'default_billing', null, 'left');

        if ($billing && $cellphoneField != $billing && $attribute != $billing && 'country_id' != $billing) {
            $customers->joinAttribute($billing, 'customer_address/' . $billing, 'default_billing', null, 'left');
        }

        if ($customers->count() == 1) {
            $customer = $customers->getFirstItem();
            if (strlen($customer->getData($cellphoneField)) > 5) {
                $customer->setData('cellphone', $this->getPrefixForCountry($customer->getCountryId()) . '-' . preg_replace('/\D/', '', $customer->getData($cellphoneField)));
            }

            return $customer;
        }

        return false;
    }

    /**
     */
    public function importCoreNewsletterSubscribers()
    {

        $news = Mage::getModel('newsletter/subscriber')
            ->getCollection()
            ->addFieldToFilter('fidelitas_updated_at', array('null' => true))
            ->addFieldToFilter('subscriber_status', 1);

        $list = Mage::getModel('fidelitas/lists')->getList();

        /** @var Mage_Newsletter_Model_Subscriber $subscriber */
        foreach ($news as $subscriber) {
            if (!$subscriber->getStoreId()) {
                continue;
            }

            $subscriber->setData('fidelitas_updated_at', new Zend_Db_Expr('NOW()'))->save();

            $data = array();

            if ($this->subscriberExists('email', $subscriber->getEmail())) {
                continue;
            }

            $data['email'] = $subscriber->getEmail();
            $data['status'] = 1;
            $data['customer_id'] = $subscriber->getCustomerId();
            $data['list'] = $list->getListnum();

            try {
                Mage::getModel('fidelitas/subscribers')->setData($data)->save();
            } catch (Exception $e) {
                Mage::logException($e);
            }
        }
    }


    /**
     * @param $field
     * @param $value
     *
     * @return bool
     */
    public function subscriberExists($field, $value)
    {

        $model = Mage::getModel('fidelitas/subscribers')
            ->getCollection()
            ->addFieldToFilter($field, $value);

        if ($model->count() == 0) {
            return false;
        }

        return $model->getFirstItem();
    }

    /**
     * @return bool|Mage_Core_Model_Abstract
     * @throws Exception
     */
    public function save()
    {
        $data = array();

        if (!$this->getEmail() || !filter_var($this->getEmail(), FILTER_VALIDATE_EMAIL)) {
            return false;
        }

        if (!Mage::getStoreConfigFlag('fidelitas/config/enabled', Mage::app()->getStore()->getId())) {
            return parent::save();
        }

        /** @var Mage_Newsletter_Model_Subscriber $coreSubscriber */
        $coreSubscriber = Mage::getModel('newsletter/subscriber')->loadByEmail($this->getEmail());
        $status = 0;
        if ($coreSubscriber->getSubscriberStatus() == 1) {
            $status = 1;
        } elseif ($coreSubscriber->getSubscriberStatus() == 2) {
            $status = 4;
        } elseif ($coreSubscriber->getSubscriberStatus() == 4) {
            $status = 0;
        } elseif ($coreSubscriber->getSubscriberStatus() == 3) {
            $status = 2;
        }

        /** @var Mage_Customer_Model_Customer $customer */
        $customer = $this->findCustomer($this->getData('email'), 'email');

        if ($customer) {
            $data['customer_id'] = $customer->getId();
            $data['birth_date'] = substr($customer->getData('dob'), 0, 10);
            $data['first_name'] = $customer->getData('firstname');
            $data['last_name'] = $customer->getData('lastname');

            if ($customer->getData('cellphone')) {
                $data['cellphone'] = $customer->getData('cellphone');
            }

            $this->addData($data);
        }

        $info = $this->subscriberExists('email', $this->getEmail());

        $model = Mage::getModel('fidelitas/egoi');
        $list = Mage::getModel('fidelitas/lists')->getList();

        if ($info) {
            $data['subscriber'] = $info->getUid();
            $this->setId($info->getId());
        } elseif ($info = $this->subscriberExists('subscriber_id', $this->getId())) {
            $data['subscriber'] = $info->getUid();
            $this->setId($info->getId());
        }

        $data['listID'] = $list->getListnum();
        $data['list_id'] = $list->getListnum();

        $extra = Mage::getModel('fidelitas/extra')->getExtra();

        $data['birth_date'] = $this->getData('birth_date');
        $indexArray[] = 'birth_date';
        $data['first_name'] = $this->getData('first_name');
        $indexArray[] = 'first_name';
        $data['last_name'] = $this->getData('last_name');
        $indexArray[] = 'last_name';
        $data['cellphone'] = $this->getData('cellphone');
        $indexArray[] = 'cellphone';

        if ($customer) {

            foreach ($extra as $element) {

                if ($customer->getData($element->getData('attribute_code'))) {
                    $data[$element->getData('extra_code')] = $customer->getData($element->getData('attribute_code'));

                    $indexArray[] = $element->getData('extra_code');
                    continue;
                }

                $billing = false;
                if (stripos($element->getData('attribute_code'), 'addr_') !== false) {
                    $attributeCode = substr($element->getData('attribute_code'), 5);
                    $billing = true;
                } else {
                    $attributeCode = $element->getData('attribute_code');
                }
                if (!$customer->getData($attributeCode)) {

                    if ($billing) {
                        $customer = $this->findCustomer($customer->getId(), 'entity_id', $attributeCode);
                    } else {
                        $customer = Mage::getModel('customer/customer')->load($customer->getId());
                    }
                }
                if ($customer->getData($attributeCode)) {

                    $data[$element->getData('extra_code')] = $customer->getData($attributeCode);

                    $indexArray[] = $element->getData('extra_code');
                } else {
                    $data[$element->getData('extra_code')] = '';
                }
            }
        } else {
            foreach ($extra as $element) {
                $data[$element->getData('extra_code')] = '';
            }
        }

        $extraStore = Mage::getModel('fidelitas/extra')
            ->getExtra()
            ->addFieldToFilter('attribute_code', 'store_id');

        if ($extraStore->getSize() == 1) {
            $extraStoreView = $extraStore->getFirstItem();

            if (!isset($data[$extraStoreView->getData('extra_code')]) ||
                $data[$extraStoreView->getData('extra_code')] == ''
            ) {
                if ($this->getStoreId()) {
                    $data[$extraStoreView->getData('extra_code')] = $this->getStoreId();
                } else {
                    /** @var Mage_Sales_Model_Order $order */
                    $order = Mage::getModel('sales/order')->getCollection()
                        ->addFieldToFilter('customer_email', $this->getEmail())
                        ->setPageSize(1)
                        ->getFirstItem();

                    if ($order->getId()) {
                        $data[$extraStoreView->getData('extra_code')] = $order->getStoreId();
                    }
                }
            }
        }

        $extraStoreCodeCol = Mage::getModel('fidelitas/extra')
            ->getExtra()
            ->addFieldToFilter('attribute_code', 'store_code');

        if ($extraStoreCodeCol->getSize() == 1) {
            $extraStoreCode = $extraStoreCodeCol->getFirstItem();
            $data[$extraStoreCode->getData('extra_code')] = Mage::app()->getStore()->getCode();

        }

        $extraStoreNameCol = Mage::getModel('fidelitas/extra')
            ->getExtra()
            ->addFieldToFilter('attribute_code', 'store_name');

        if ($extraStoreNameCol->getSize() == 1) {
            $extraStoreName= $extraStoreNameCol->getFirstItem();
            $data[$extraStoreName->getData('extra_code')] = Mage::app()->getStore()->getName();

        }

        $model->addData($data);
        $model->addData($this->getData());
        $this->addData($data);

        foreach ($this->getData() as $key => $value) {
            if (!is_scalar($value)) {
                $this->unsetData($key);
            }
        }

        foreach ($model->getData() as $key => $value) {
            if (!is_scalar($value)) {
                $model->unsetData($key);
            }
        }

        if (!$this->getEmail() || !filter_var($this->getEmail(), FILTER_VALIDATE_EMAIL)) {
            return false;
        }

        if ($this->getData('inCron') === true && !Mage::registry('fidelitas_first_run')) {
            return parent::save();
        }

        try {
            $model->unsetData('subscriber');

            $model->setData('status', $status);

            if ($list->getListnum()) {


                if ($this->getId()) {

                    if (strlen($this->getData('subscriber')) > 1) {

                        $model->setData('subscriber', $this->getData('subscriber'));
                        $model->setData('uid', $this->getData('subscriber'));


                        $result = $model->editSubscriber();

                        if (isset($result['uid'])) {
                            $this->setData('uid', $result->getData('uid'));
                        }
                    }

                } else {

                    $result = $model->addSubscriber();

                    if (isset($result['uid'])) {
                        $this->setData('uid', $result->getData('uid'));
                        if (!Mage::app()->getStore()->isAdmin()) {
                            Mage::getSingleton('core/cookie')->set('egoi-subscriber', $result['uid'], true);
                        }
                    }

                }
            }

        } catch (Exception $e) {
            Mage::logException($e);
            return false;
        }

        return parent::save();
    }


    /**
     * @return Mage_Core_Model_Abstract
     * @throws Exception
     */
    public function delete()
    {
        $model = Mage::getModel('fidelitas/egoi');

        $data = array();
        $data['listID'] = $this->getList();
        $data['subscriber'] = $this->getUid();

        if (!$this->getData('inCron') && !Mage::registry('fidelitas_clean')) {
            try {

                $model->setData($data)->removeSubscriber();
            } catch (Exception $e) {
                Mage::logException($e);
            }

            Mage::getModel('newsletter/subscriber')->loadByEmail($data['email'])->delete();
        }

        return parent::delete();
    }

    /**
     * @param $event
     *
     * @return bool|void
     */
    public function updateFromNewsletterCore($event)
    {
        /** @var Mage_Newsletter_Model_Subscriber $subscriber */
        $subscriber = $event->getDataObject();
        $email = $subscriber->getSubscriberEmail();

        if ($subscriber->getId()) {
            $subscriber->setImportMode(true);
        }
        $subscriber->setData('fidelitas_updated_at', new Zend_Db_Expr('NOW()'));

        if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
            return false;
        }

        if (!$subscriber->getIsStatusChanged() &&
            $subscriber->getOrigData('subscriber_status') == $subscriber->getData('subscriber_status')
        ) {
            return;
        }

        try {
            $status = 0;
            if ($subscriber->getSubscriberStatus() == 1) {
                $status = 1;
            } elseif ($subscriber->getSubscriberStatus() == 2) {
                $status = 4;
            } elseif ($subscriber->getSubscriberStatus() == 4) {
                $status = 0;
            } elseif ($subscriber->getSubscriberStatus() == 3) {
                $status = 2;
            }

            $data = array();
            $data['email'] = $email;
            $data['status'] = $status;
            $data['store_id'] = $subscriber->getStoreId();

            return $this->addData($data)->save();

        } catch (Exception $e) {
            Mage::logException($e);
        }

    }

    /**
     * @return array
     */
    public static function getPhonePrefixs()
    {
        $phones = self::phonePrefixsList();

        $return = array();
        $return[''] = Mage::helper('fidelitas')->__('-- Please Choose --');
        foreach ($phones as $value) {
            $return[$value[2]] = ucwords(strtolower($value[0])) . ' (+' . $value[2] . ')';
        }

        asort($return);

        return $return;
    }

    /**
     * @param $countryCode
     *
     * @return string
     */
    public function getPrefixForCountry($countryCode)
    {

        $phones = self::phonePrefixsList();
        foreach ($phones as $phone) {

            if ($phone[1] == $countryCode) {
                return $phone[2];
            }
        }

        return '';
    }

    /**
     * @return array
     */
    public static function phonePrefixsList()
    {

        return array(array('CANADA', 'CA', '1'),
            array('PUERTO RICO', 'PR', '1'),
            array('UNITED STATES', 'US', '1'),
            array('ARMENIA', 'AM', '7'),
            array('KAZAKHSTAN', 'KZ', '7'),
            array('RUSSIAN FEDERATION', 'RU', '7'),
            array('EGYPT', 'EG', '20'),
            array('SOUTH AFRICA (Zuid Afrika)', 'ZA', '27'),
            array('GREECE', 'GR', '30'),
            array('NETHERLANDS', 'NL', '31'),
            array('BELGIUM', 'BE', '32'),
            array('FRANCE', 'FR', '33'),
            array('SPAIN (España)', 'ES', '34'),
            array('HUNGARY', 'HU', '36'),
            array('ITALY', 'IT', '39'),
            array('ROMANIA', 'RO', '40'),
            array('SWITZERLAND (Confederation of Helvetia)', 'CH', '41'),
            array('AUSTRIA', 'AT', '43'),
            array('GREAT BRITAIN (United Kingdom)', 'GB', '44'),
            array('UNITED KINGDOM', 'GB', '44'),
            array('DENMARK', 'DK', '45'),
            array('SWEDEN', 'SE', '46'),
            array('NORWAY', 'NO', '47'),
            array('POLAND', 'PL', '48'),
            array('GERMANY (Deutschland)', 'DE', '49'),
            array('PERU', 'PE', '51'),
            array('MEXICO', 'MX', '52'),
            array('CUBA', 'CU', '53'),
            array('ARGENTINA', 'AR', '54'),
            array('BRAZIL', 'BR', '55'),
            array('CHILE', 'CL', '56'),
            array('COLOMBIA', 'CO', '57'),
            array('VENEZUELA', 'VE', '58'),
            array('MALAYSIA', 'MY', '60'),
            array('AUSTRALIA', 'AU', '61'),
            array('INDONESIA', 'ID', '62'),
            array('PHILIPPINES', 'PH', '63'),
            array('NEW ZEALAND', 'NZ', '64'),
            array('SINGAPORE', 'SG', '65'),
            array('THAILAND', 'TH', '66'),
            array('JAPAN', 'JP', '81'),
            array('KOREA (Republic of [South] Korea)', 'KR', '82'),
            array('VIET NAM', 'VN', '84'),
            array('CHINA', 'CN', '86'),
            array('TURKEY', 'TR', '90'),
            array('INDIA', 'IN', '91'),
            array('PAKISTAN', 'PK', '92'),
            array('AFGHANISTAN', 'AF', '93'),
            array('SRI LANKA (formerly Ceylon)', 'LK', '94'),
            array('MYANMAR (formerly Burma)', 'MM', '95'),
            array('IRAN (Islamic Republic of Iran)', 'IR', '98'),
            array('MOROCCO', 'MA', '212'),
            array('ALGERIA (El Djazaïr)', 'DZ', '213'),
            array('TUNISIA', 'TN', '216'),
            array('LIBYA (Libyan Arab Jamahirya)', 'LY', '218'),
            array('GAMBIA, THE', 'GM', '220'),
            array('SENEGAL', 'SN', '221'),
            array('MAURITANIA', 'MR', '222'),
            array('MALI', 'ML', '223'),
            array('GUINEA', 'GN', '224'),
            array('CÔTE D\'IVOIRE (Ivory Coast)', 'CI', '225'),
            array('BURKINA FASO', 'BF', '226'),
            array('NIGER', 'NE', '227'),
            array('TOGO', 'TG', '228'),
            array('BENIN', 'BJ', '229'),
            array('MAURITIUS', 'MU', '230'),
            array('LIBERIA', 'LR', '231'),
            array('SIERRA LEONE', 'SL', '232'),
            array('GHANA', 'GH', '233'),
            array('NIGERIA', 'NG', '234'),
            array('CHAD (Tchad)', 'TD', '235'),
            array('CENTRAL AFRICAN REPUBLIC', 'CF', '236'),
            array('CAMEROON', 'CM', '237'),
            array('CAPE VERDE', 'CV', '238'),
            array('SAO TOME AND PRINCIPE', 'ST', '239'),
            array('EQUATORIAL GUINEA', 'GQ', '240'),
            array('GABON', 'GA', '241'),
            array('CONGO, REPUBLIC OF', 'CG', '242'),
            array('CONGO, THE DEMOCRATIC REPUBLIC OF THE (formerly Zaire)', 'CD', '243'),
            array('ANGOLA', 'AO', '244'),
            array('GUINEA-BISSAU', 'GW', '245'),
            array('ASCENSION ISLAND', '', '247'),
            array('SEYCHELLES', 'SC', '248'),
            array('SUDAN', 'SD', '249'),
            array('RWANDA', 'RW', '250'),
            array('ETHIOPIA', 'ET', '251'),
            array('SOMALIA', 'SO', '252'),
            array('DJIBOUTI', 'DJ', '253'),
            array('KENYA', 'KE', '254'),
            array('TANZANIA', 'TZ', '255'),
            array('UGANDA', 'UG', '256'),
            array('BURUNDI', 'BI', '257'),
            array('MOZAMBIQUE (Moçambique)', 'MZ', '258'),
            array('ZAMBIA (formerly Northern Rhodesia)', 'ZM', '260'),
            array('MADAGASCAR', 'MG', '261'),
            array('RÉUNION', 'RE', '262'),
            array('ZIMBABWE', 'ZW', '263'),
            array('NAMIBIA', 'NA', '264'),
            array('MALAWI', 'MW', '265'),
            array('LESOTHO', 'LS', '266'),
            array('BOTSWANA', 'BW', '267'),
            array('SWAZILAND', 'SZ', '268'),
            array('COMOROS', 'KM', '269'),
            array('MAYOTTE', 'YT', '269'),
            array('SAINT HELENA', 'SH', '290'),
            array('ERITREA', 'ER', '291'),
            array('ARUBA', 'AW', '297'),
            array('FAEROE ISLANDS', 'FO', '298'),
            array('GREENLAND', 'GL', '299'),
            array('GIBRALTAR', 'GI', '350'),
            array('PORTUGAL', 'PT', '351'),
            array('LUXEMBOURG', 'LU', '352'),
            array('IRELAND', 'IE', '353'),
            array('ICELAND', 'IS', '354'),
            array('ALBANIA', 'AL', '355'),
            array('MALTA', 'MT', '356'),
            array('CYPRUS', 'CY', '357'),
            array('FINLAND', 'FI', '358'),
            array('BULGARIA', 'BG', '359'),
            array('LITHUANIA', 'LT', '370'),
            array('LATVIA', 'LV', '371'),
            array('ESTONIA', 'EE', '372'),
            array('MOLDOVA', 'MD', '373'),
            array('BELARUS', 'BY', '375'),
            array('ANDORRA', 'AD', '376'),
            array('MONACO', 'MC', '377'),
            array('SAN MARINO (Republic of)', 'SM', '378'),
            array('VATICAN CITY (Holy See)', 'VA', '379'),
            array('UKRAINE', 'UA', '380'),
            array('SERBIA (Republic of Serbia)', 'RS', '381'),
            array('MONTENEGRO', 'ME', '382'),
            array('CROATIA (Hrvatska)', 'HR', '385'),
            array('SLOVENIA', 'SI', '386'),
            array('BOSNIA AND HERZEGOVINA', 'BA', '387'),
            array('MACEDONIA (Former Yugoslav Republic of Macedonia)', 'MK', '389'),
            array('CZECH REPUBLIC', 'CZ', '420'),
            array('SLOVAKIA (Slovak Republic)', 'SK', '421'),
            array('LIECHTENSTEIN (Fürstentum Liechtenstein)', 'LI', '423'),
            array('FALKLAND ISLANDS (MALVINAS)', 'FK', '500'),
            array('BELIZE', 'BZ', '501'),
            array('GUATEMALA', 'GT', '502'),
            array('EL SALVADOR', 'SV', '503'),
            array('HONDURAS', 'HN', '504'),
            array('NICARAGUA', 'NI', '505'),
            array('COSTA RICA', 'CR', '506'),
            array('PANAMA', 'PA', '507'),
            array('SAINT PIERRE AND MIQUELON', 'PM', '508'),
            array('HAITI', 'HT', '509'),
            array('GUADELOUPE', 'GP', '590'),
            array('BOLIVIA', 'BO', '591'),
            array('GUYANA', 'GY', '592'),
            array('ECUADOR', 'EC', '593'),
            array('FRENCH GUIANA', 'GF', '594'),
            array('PARAGUAY', 'PY', '595'),
            array('MARTINIQUE', 'MQ', '596'),
            array('SURINAME', 'SR', '597'),
            array('URUGUAY', 'UY', '598'),
            array('BONAIRE, ST. EUSTATIUS, AND SABA', 'BQ', '599'),
            array('CURAÃ‡AO', 'CW', '599'),
            array('NETHERLANDS ANTILLES (obsolete)', 'AN', '599'),
            array('SINT MAARTEN', 'SX', '599'),
            array('TIMOR-LESTE (formerly East Timor)', 'TL', '670'),
            array('BRUNEI DARUSSALAM', 'BN', '673'),
            array('NAURU', 'NR', '674'),
            array('PAPUA NEW GUINEA', 'PG', '675'),
            array('TONGA', 'TO', '676'),
            array('SOLOMON ISLANDS', 'SB', '677'),
            array('VANUATU', 'VU', '678'),
            array('FIJI', 'FJ', '679'),
            array('PALAU', 'PW', '680'),
            array('WALLIS AND FUTUNA', 'WF', '681'),
            array('COOK ISLANDS', 'CK', '682'),
            array('NIUE', 'NU', '683'),
            array('SAMOA (formerly Western Samoa)', 'WS', '685'),
            array('KIRIBATI', 'KI', '686'),
            array('NEW CALEDONIA', 'NC', '687'),
            array('TUVALU', 'TV', '688'),
            array('FRENCH POLYNESIA', 'PF', '689'),
            array('TOKELAU', 'TK', '690'),
            array('MICRONESIA (Federated States of Micronesia)', 'FM', '691'),
            array('MARSHALL ISLANDS', 'MH', '692'),
            array('KOREA (Democratic Peoples Republic of [North] Korea)', 'KP', '850'),
            array('HONG KONG (Special Administrative Region of China)', 'HK', '852'),
            array('MACAO (Special Administrative Region of China)', 'MO', '853'),
            array('CAMBODIA', 'KH', '855'),
            array('LAO PEOPLE\'S DEMOCRATIC REPUBLIC', 'LA', '856'),
            array('BANGLADESH', 'BD', '880'),
            array('TAIWAN (Chinese Taipei for IOC)', 'TW', '886'),
            array('MALDIVES', 'MV', '960'),
            array('LEBANON', 'LB', '961'),
            array('JORDAN (Hashemite Kingdom of Jordan)', 'JO', '962'),
            array('SYRIAN ARAB REPUBLIC', 'SY', '963'),
            array('IRAQ', 'IQ', '964'),
            array('KUWAIT', 'KW', '965'),
            array('SAUDI ARABIA (Kingdom of Saudi Arabia)', 'SA', '966'),
            array('YEMEN (Yemen Arab Republic)', 'YE', '967'),
            array('OMAN', 'OM', '968'),
            array('PALESTINIAN TERRITORIES', 'PS', '970'),
            array('UNITED ARAB EMIRATES', 'AE', '971'),
            array('ISRAEL', 'IL', '972'),
            array('BAHRAIN', 'BH', '973'),
            array('QATAR', 'QA', '974'),
            array('BHUTAN', 'BT', '975'),
            array('MONGOLIA', 'MN', '976'),
            array('NEPAL', 'NP', '977'),
            array('TAJIKISTAN', 'TJ', '992'),
            array('TURKMENISTAN', 'TM', '993'),
            array('AZERBAIJAN', 'AZ', '994'),
            array('KYRGYZSTAN', 'KG', '996'),
            array('UZBEKISTAN', 'UZ', '998'),
            array('BAHAMAS', 'BS', '1242'),
            array('BARBADOS', 'BB', '1246'),
            array('ANGUILLA', 'AI', '1264'),
            array('ANTIGUA AND BARBUDA', 'AG', '1268'),
            array('VIRGIN ISLANDS, BRITISH', 'VG', '1284'),
            array('VIRGIN ISLANDS, U.S.', 'VI', '1340'),
            array('CAYMAN ISLANDS', 'KY', '1345'),
            array('BERMUDA', 'BM', '1441'),
            array('GRENADA', 'GD', '1473'),
            array('TURKS AND CAICOS ISLANDS', 'TC', '1649'),
            array('MONTSERRAT', 'MS', '1664'),
            array('NORTHERN MARIANA ISLANDS', 'MP', '1670'),
            array('GUAM', 'GU', '1671'),
            array('AMERICAN SAMOA', 'AS', '1684'),
            array('SAINT LUCIA', 'LC', '1758'),
            array('DOMINICA', 'DM', '1767'),
            array('SAINT VINCENT AND THE GRENADINES', 'VC', '1784'),
            array('DOMINICAN REPUBLIC', 'DO', '1809'),
            array('TRINIDAD AND TOBAGO', 'TT', '1868'),
            array('SAINT KITTS AND NEVIS', 'KN', '1869'),
            array('JAMAICA', 'JM', '1-876'));
    }


    /**
     *
     */
    public function sync()
    {

        $core = Mage::getModel('newsletter/subscriber')
            ->getCollection()
            ->addFieldToFilter('subscriber_status', 1);

        /** @var Mage_Newsletter_Model_Subscriber $susbcriber */
        foreach ($core as $susbcriber) {

            /** @var Mage_Customer_Model_Customer $customer */
            $customer = Mage::getModel('customer/customer')->load($susbcriber->getCustomerId());
            $egoi = Mage::getModel('fidelitas/subscribers')->load($susbcriber->getEmail(), 'email');

            if ($customer->getId() && !$egoi->getCustomerId()) {

                $data['email'] = $customer->getEmail();
                $data['customer_id'] = $customer->getId();
                $data['birth_date'] = substr($customer->getData('dob'), 0, 10);
                $data['first_name'] = $customer->getData('firstname');
                $data['last_name'] = $customer->getData('lastname');

                if ($customer->getData('cellphone')) {
                    $data['cellphone'] = $customer->getData('cellphone');
                }

                $egoi->addData($data)->save();
            }


        }
    }
}
